;**************************************************************************
;
;                           PVK40 Example : Blinking
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on B3 LED on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
portb   equ     0xF81
portd   equ     0xF83
fsr0l   equ     0xFE9
fsr0h   equ     0xFEA
indf0   equ     0xFEF
rcon    equ     0xFD0
intcon  equ     0xFF2
t0con   equ     0xFD5
tmr0l   equ     0xFD6
tmr0h   equ     0xFD7
; Destination definition :
w       equ     0x00
f       equ     0x01
fast    equ     0x01            ;fast return
; RAM access bit definition :
a       equ     0x00            ;access RAM
banked  equ     0x01            ;bank specified by BSR register
; Bits definition :
#define LED     portb,3         ;LED is on the RB3 pin
#define PB      portd,0         ;pushbutton 0 is on the RD0 pin
#define IPEN    rcon,7          ;priority interrupt bit
#define GIE     intcon,7        ;Global Interrupt Enable bit
#define TMR0IF  intcon,2        ;TMR0 interrupt flag
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x08            ;interrupt vector
        bcf     TMR0IF,a        ;clear flag
        btg     LED,a           ;change LED state
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0x0F            ;bank 15
        movwf   fsr0h,a
        movlw   0x93            ;0x93 = trisb address
        movwf   fsr0l,a
        movlw   B'11110111'     ;pin RB3 is output
        movwf   indf0,a         ;write to trisb
        movlw   0x95            ;0x95 = trisd address
        movwf   fsr0l,a
        movlw   B'11111111'     ;portd pins are inputs
        movwf   indf0,a         ;write to trisd
;
        movlw   B'10000010'     ;timer0 enabled
        movwf   t0con,a
        clrf    tmr0h,a
        clrf    tmr0l,a
        bcf     IPEN,a          ;disable priority interrupt
        movlw   B'00100000'     ;unmask interrupt from TMR0
        movwf   intcon,a
        bsf     GIE,a           ;enable global interrupt
;
Main:   btfsc   PB,a            ;test pushbutton
        goto    Main
        bcf     GIE,a           ;if pressed, disable interrupt
Main_B: btfss   PB,a            ;wait for release
        goto    Main_B
        bsf     GIE,a           ;enable interrupt
        goto    Main            ;closes the loop
;**************************************************************************
; end of PVK40 Example : Blinking
        end
