;**************************************************************************
;
;                           PVK40 Example : Counter
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
tmr0    equ     0x01            ;timer 0
optreg  equ     0x01            ;direct addressing
pcl     equ     0x02
status  equ     0x03            ;status is on the 0x03 address
portb   equ     0x06
trisb   equ     0x06            ;direct addressing
portd   equ     0x08
trisd   equ     0x08            ;direct addressing
intcon  equ     0x0B
; General Purpose Register definition :
intcnt  equ     0x20            ;interrupt counting
cnt1    equ     0x21            ;used for Wait subroutine
dig1    equ     0x22            ;digit '1'
dig2    equ     0x23            ;digit '10'
dig3    equ     0x24            ;digit '100'
dig4    equ     0x25            ;digit '1000'
bckwreg equ     0x26            ;working register backup during interrupt
; Destination definition :
w       equ     0x00
f       equ     0x01
; Bits definition :
#define RP0     status,5        ;RP0 is bit 5 in status register
#define Z       status,2        ;Z is bit 2 in status register
#define T0IF    intcon,2        ;TMR0 interrupt flag
#define GIE     intcon,7        ;Global Interrupt Enable bit
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        bcf     T0IF            ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_D           ;then return
;
        movlw   0x19            ;else reinitialise intcnt register
        movwf   intcnt
        incf    dig1,f          ;increment dig1
        movlw   0x0A
        xorwf   dig1,w          ;if dig1 > 9 
        btfss   Z
        goto    Int_A
        clrf    dig1            ;then dig1 = 0
        incf    dig2,f          ;and increment dig2
Int_A:  movlw   0x0A
        xorwf   dig2,w          ;if dig2 > 9 
        btfss   Z
        goto    Int_B
        clrf    dig2            ;then dig2 = 0
        incf    dig3,f          ;and increment dig3
Int_B:  movlw   0x0A
        xorwf   dig3,w          ;if dig3 > 9 
        btfss   Z
        goto    Int_C
        clrf    dig3            ;then dig3 = 0
        incf    dig4,f          ;and increment dig4
Int_C:  movlw   0x0A
        xorwf   dig4,w          ;if dig4 > 9 then dig4 = 0
        btfsc   Z
        clrf    dig4
Int_D:  movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0xFF
        movwf   portb           ;set portb
        movwf   portd           ;set portd
        bsf     RP0             ;bank 1
        movlw   B'11101000'     ;pins RB0,1,2,4 are outputs
        movwf   trisb
        clrf    trisd           ;portd pins are outputs
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   optreg
        bcf     RP0             ;bank 0
        clrf    dig1
        clrf    dig2
        clrf    dig3
        clrf    dig4
        movlw   0x19            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     GIE             ;enable global interrupt
;
Main:   movf    dig1,w
        call    Table
        movwf   portd
        bcf     DCOM1           ;turn on display 1
        call    Wait
        bsf     DCOM1           ;turn off display 1
        movf    dig2,w
        call    Table
        movwf   portd
        bcf     DCOM2           ;turn on display 2
        call    Wait
        bsf     DCOM2           ;turn off display 2
        movf    dig3,w
        call    Table
        movwf   portd
        bcf     DCOM3           ;turn on display 3
        call    Wait
        bsf     DCOM3           ;turn off display 3
        movf    dig4,w
        call    Table
        movwf   portd
        bcf     DCOM4           ;turn on display 4
        call    Wait
        bsf     DCOM4           ;turn off display 4
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;**************************************************************************
; end of PVK40 Example : Counter
        end
