;**************************************************************************
;
;                           PVK40 Example : Counter
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
portb   equ     0xF81
trisb   equ     0xF93
portd   equ     0xF83
trisd   equ     0xF95
status  equ     0xFD8
wreg    equ     0xFE8
pcl     equ     0xFF9
rcon    equ     0xFD0
intcon  equ     0xFF2
t0con   equ     0xFD5
tmr0l   equ     0xFD6
tmr0h   equ     0xFD7
; General Purpose Register definition :
cnt1    equ     0x00            ;used for Wait subroutine
intcnt  equ     0x01            ;interrupt counting
dig1    equ     0x02            ;digit '1'
dig2    equ     0x03            ;digit '10'
dig3    equ     0x04            ;digit '100'
dig4    equ     0x05            ;digit '1000'
; Destination definition :
w       equ     0x00
f       equ     0x01
fast    equ     0x01            ;fast return
; RAM access bit definition :
a       equ     0x00            ;access RAM
banked  equ     0x01            ;bank specified by BSR register
; Bits definition :
#define Z       status,2        ;Z is bit 2 in status register
#define TMR0IF  intcon,2        ;TMR0 interrupt flag
#define GIE     intcon,7        ;Global Interrupt Enable bit
#define IPEN    rcon,7          ;priority interrupt bit
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x08            ;interrupt vector
        bcf     TMR0IF,a        ;clear flag
        decfsz  intcnt,f,a      ;if intcnt > 1
        retfie  fast            ;then return
;
        movlw   0x19            ;else reinitialise intcnt register
        movwf   intcnt,a
        incf    dig1,f,a        ;increment dig1
        movlw   0x0A
        xorwf   dig1,w,a        ;if dig1 > 9 
        bnz     Int_A
        clrf    dig1,a          ;then dig1 = 0
        incf    dig2,f,a        ;and increment dig2
Int_A:  movlw   0x0A
        xorwf   dig2,w,a        ;if dig2 > 9 
        bnz     Int_B
        clrf    dig2,a          ;then dig2 = 0
        incf    dig3,f,a        ;and increment dig3
Int_B:  movlw   0x0A
        xorwf   dig3,w,a        ;if dig3 > 9 
        bnz     Int_C
        clrf    dig3,a          ;then dig3 = 0
        incf    dig4,f,a        ;and increment dig4
Int_C:  movlw   0x0A
        xorwf   dig4,w,a        ;if dig4 > 9 then dig4 = 0
        btfsc   Z,a
        clrf    dig4,a
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
Start:  setf    portb,a         ;set portb
        setf    portd,a         ;set portd
        movlw   B'11101000'     ;pins RB0,1,2,4 are outputs
        movwf   trisb,a
        clrf    trisd,a         ;portd pins are outputs
        movlw   B'11000110'     ;prescaler assigned to TMR0
        movwf   t0con,a
        clrf    dig1,a
        clrf    dig2,a
        clrf    dig3,a
        clrf    dig4,a
        movlw   0x19            ;initialise intcnt register
        movwf   intcnt,a
        clrf    tmr0l,a
        bcf     IPEN,a          ;disable priority interrupt
        movlw   B'00100000'     ;unmask interrupt from TMR0
        movwf   intcon,a
        bsf     GIE,a           ;enable global interrupt
;
Main:   movf    dig1,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM1,a         ;turn on display 1
        call    Wait
        bsf     DCOM1,a         ;turn off display 1
        movf    dig2,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM2,a         ;turn on display 2
        call    Wait
        bsf     DCOM2,a         ;turn off display 2
        movf    dig3,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM3,a         ;turn on display 3
        call    Wait
        bsf     DCOM3,a         ;turn off display 3
        movf    dig4,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM4,a         ;turn on display 4
        call    Wait
        bsf     DCOM4,a         ;turn off display 4
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  rlncf   wreg,f,a        ;Display segments table
        addwf   pcl,f,a         ;wreg * 2 -> pcl
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1,a
Wait_B: decfsz  cnt1,f,a        ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;**************************************************************************
; end of PVK40 Example : Counter
        end
