;**************************************************************************
;
;                           PVK40 Example : A/D Converter
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on PHOTO on S9
; 4) Switch on B3 LED on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
cnt1    equ     0x20            ;used for Wait subroutine
; Bits definition :
#define LED     portb,3         ;LED is on the RB3 pin
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  bcf     LED
        bsf     status,RP0      ;bank 1
        movlw   B'11110111'     ;pin RB3 is output
        movwf   trisb
        movlw   B'00000000'     ;pin RA1 is analog input
        movwf   adcon1
        bcf     status,RP0      ;bank 0
        movlw   B'00110001'     ;enable TMR1
        movwf   t1con
        movlw   B'01001001'     ;enable A/D converter module
        movwf   adcon0
        call    Wait            ;A/D acquisition time
;
Main:   bsf     adcon0,GO       ;start conversion
Adw:    btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw
;
        bsf     LED             ;turn on the LED
        clrf    tmr1l
        bcf     pir1,TMR1IF
        movf    adresh,w        ;preset tmr1
        movwf   tmr1h
T1w_A:  btfss   pir1,TMR1IF     ;wait for TMR1 overflow
        goto    T1w_A
;
        bcf     LED             ;turn off the LED
        clrf    tmr1l
        bcf     pir1,TMR1IF
        movf    adresh,w        ;preset tmr1
        movwf   tmr1h
T1w_B:  btfss   pir1,TMR1IF     ;wait for TMR1 overflow
        goto    T1w_B
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;**************************************************************************
; end of PVK40 Example : A/D Converter
        end
