;**************************************************************************
;
;                           PVK40 Example : Analogue comparator
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on TRIM1 on S9
; 4) Switch on TRIM2 on S9
; 5) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
cnt1    equ     0x20            ;used in Wait subroutine
bckwreg equ     0x21            ;working register backup during interrupt
dig1    equ     0x22            ;digit '1'
dig2    equ     0x23            ;digit '10'
intcnt  equ     0x24            ;interrupt counting
tmph    equ     0x25            ;used for comparsion
tmpl    equ     0x26            ;used for comparsion
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        bcf     intcon,T0IF     ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Intrt           ;then return
;
        movlw   0x0B            ;else reinitialise intcnt register
        movwf   intcnt
        bsf     adcon0,GO       ;start A/D conversion
Adw1:   btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw1
        bsf     status,RP0      ;bank 1
        movf    adresl,W        ;copy adresh/l to tmph/l
        bcf     status,RP0      ;bank 0
        movwf   tmpl
        movf    adresh,W
        movwf   tmph
        bsf     adcon0,CHS0     ;analog channel = RA3
        call    Wait            ;A/D acquisition time
        bsf     adcon0,GO       ;start A/D conversion
Adw2:   btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw2
        bcf     adcon0,CHS0     ;analog channel = RA2
        movf    adresh,W        ;compare tmph/l with adresh/l
        subwf   tmph,W          ;tmph - adresh
        btfss   status,C
        goto    Dilo            ;tmph < adresh
        btfss   status,Z
        goto    Dihi            ;tmph > adresh
        bsf     status,RP0      ;bank 1
        movf    adresl,W
        bcf     status,RP0      ;bank 0
        subwf   tmpl,W          ;tmpl - adresl
        btfss   status,C
        goto    Dilo            ;tmpl < adresl
        btfss   status,Z
        goto    Dihi            ;tmpl > adresl
;
Dieq:   movlw   0x04            ;equal
        movwf   dig2
        movwf   dig1
        goto    Intrt 
Dihi:   movlw   0x00            ;Higher
        movwf   dig2
        addlw   0x02
        movwf   dig1
        goto    Intrt
Dilo:   movlw   0x01            ;lower
        movwf   dig2
        addlw   0x02
        movwf   dig1
Intrt:  movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0xFF
        movwf   portb           ;set portb
        movwf   portd           ;set portd
        bsf     status,RP0      ;bank 1
        movlw   B'10000000'     ;pins RA2,3 are analog inputs
        movwf   adcon1
        movlw   B'11111100'     ;pins RB0,1 are outputs
        movwf   trisb
        clrf    trisd           ;portd pins are outputs
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   option_reg
        bcf     status,RP0      ;bank 0
        movlw   0x05
        movwf   dig1
        movwf   dig2
        movlw   B'01010001'     ;enable A/D converter module
        movwf   adcon0          ;analog channel = RA2
        movlw   0x0B            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
;
Main:   movf    dig1,w
        call    Table
        movwf   portd
        bcf     DCOM1           ;turn on display 1
        call    Wait
        call    Wait
        bsf     DCOM1           ;turn off display 1
        movf    dig2,w
        call    Table
        movwf   portd
        bcf     DCOM2           ;turn on display 2
        call    Wait
        call    Wait
        bsf     DCOM2           ;turn off display 2
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'10001001'     ;H
        retlw   B'11000111'     ;L
        retlw   B'11101111'     ;i
        retlw   B'10100011'     ;o
        retlw   B'10111111'     ;-
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;**************************************************************************
; end of PVK40 Example : Analogue comparator
        end
