;**************************************************************************
;
;                       PVK40 Example : Serial EEPROM 24xx (HW)
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board :
; 1) Insert 24LC01B into 24xx socket
; 2) Switch off all DIP switches except:
; 3) Switch on OSC 3.276M on S9
; 4) Switch on B0,1,2,4 DISP on S11
; 5) Switch on B3 LED on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
cnt1    equ     0x00            ;used in Wait subroutine
dig1    equ     0x01            ;digit '1'
dig2    equ     0x02            ;digit '10'
dig3    equ     0x03            ;digit '100'
dig4    equ     0x04            ;digit '1000'
digtmp1 equ     0x05            ;store dig1 value
digtmp2 equ     0x06            ;store dig2 value
digtmp3 equ     0x07            ;store dig2 value
eedatb  equ     0x08            ;EEPROM data write/read
intmp   equ     0x09            ;used in Input subroutine
eeadrs  equ     0x0A            ;used in Wree and Rdee subroutines
swrtmp  equ     0x0B            ;used in Swree subroutine
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
#define LED     portb,3         ;LED
#define PB0     portd,0         ;pushbutton 0
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x08            ;interrupt vector
        bcf     intcon,T0IF,a   ;clear flag
        incf    dig1,f,a        ;increment dig1
        movlw   0x09
        cpfsgt  dig1,a
        bra     Int_A
        clrf    dig1,a          ;if dig1 > 9 then dig1 = 0
        incf    dig2,f,a        ;and increment dig2
Int_A:  movlw   0x09
        cpfsgt  dig2,a
        bra     Int_B
        clrf    dig2,a          ;if dig2 > 9 then dig2 = 0
        incf    dig3,f,a        ;and increment dig3
Int_B:  movlw   0x09
        cpfsgt  dig3,a
        bra     Int_C
        clrf    dig3,a          ;if dig3 > 9 then dig3 = 0
        incf    dig4,f,a        ;and increment dig4
Int_C:  movlw   0x0A
        cpfslt  dig4,a
        clrf    dig4,a
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   B'11110111'     ;clear RB3 pin latch
        movwf   portb,a
        setf    portd,a         ;set portd latch
        movlw   B'11100000'     ;pins RB0,1,2,3,4 are outputs
        movwf   trisb,a
        clrf    trisd,a         ;portd pins are outputs
        clrf    sspstat,a
        bsf     sspstat,SMP,a   ;disable slew rate control
        movlw   0x02
        movwf   sspadd,a        ;setup 400 kHz I2C clock (273kHz)
        clrf    sspcon2,a
        movlw   B'00101000'     ;I2C Master mode
        movwf   sspcon1,a       ;SSPEN enabled
        bcf     pir1,SSPIF,a    ;clear SSP interrupt flag
;
        movlw   0x00            ;read address 00
        call    Rdee            ;from EEPROM
        movf    eedatb,w,a
        andlw   0x0F
        movwf   dig4,a          ;save to dig4
        movlw   0x01            ;read address 01
        call    Rdee            ;from EEPROM
        movf    eedatb,w,a
        andlw   0x0F
        movwf   dig3,a          ;save to dig3
        movlw   0x02            ;read address 02
        call    Rdee            ;from EEPROM
        movf    eedatb,w,a
        andlw   0x0F
        movwf   dig2,a          ;save to dig2
        movlw   0x03            ;read address 03
        call    Rdee            ;from EEPROM
        movf    eedatb,w,a
        andlw   0x0F
        movwf   dig1,a          ;save to dig1
;
        movlw   B'10000001'     ;prescaler 1:4 assigned to TMR0
        movwf   t0con,a
        clrf    tmr0h,a
        clrf    tmr0l,a
        bcf     rcon,IPEN,a     ;disable priority interrupt
        movlw   B'00100000'     ;unmask interrupt from TMR0
        movwf   intcon,a
        bsf     intcon,GIE,a    ;enable global interrupt
;
Main:   movf    dig1,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM1,a         ;turn on display 1
        call    Wait
        bsf     DCOM1,a         ;turn off display 1
        movf    dig2,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM2,a         ;turn on display 2
        call    Wait
        bsf     DCOM2,a         ;turn off display 2
        movf    dig3,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM3,a         ;turn on display 3
        call    Wait
        bsf     DCOM3,a         ;turn off display 3
        movf    dig4,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM4,a         ;turn on display 4
        call    Wait
        bsf     DCOM4,a         ;turn off display 4
;
        bsf     trisd,0,a       ;RD0 is input
        call    Wait            ;delay to stable inputs
        btfss   PB0,a           ;if pushbutton is pressed
        bsf     LED,a           ;turn on the LED
        btfsc   PB0,a
        bcf     LED,a
        bcf     trisd,0,a       ;RD0 is output
;
        btfss   LED,a           ;if the LED is on
        goto    Main
        bcf     intcon,GIE,a    ;disable interrupt
        movff   dig1,digtmp1    ;save dig1-dig4 state
        movff   dig2,digtmp2
        movff   dig3,digtmp3
        movff   dig4,eedatb
        bsf     intcon,GIE,a    ;enable interrupt
        movlw   0x00
        call    Swree           ;write to EEPROM
        movff   digtmp3,eedatb
        movlw   0x01
        call    Swree           ;write to EEPROM
        movff   digtmp2,eedatb
        movlw   0x02
        call    Swree           ;write to EEPROM
        movff   digtmp1,eedatb
        movlw   0x03
        call    Swree           ;write to EEPROM
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        bsf     pclath,0,a      ;subroutine position = 0x0001xx
        rlncf   wreg,f,a        ;wreg * 2 -> pcl
        addwf   pcl,f,a
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1,a
Wait_B: decfsz  cnt1,f,a        ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Swree:  movwf   eeadrs,a        ;this subroutine writes to EEPROM
        movff   eedatb,swrtmp   ;use this subroutine to avoid multiply
        movf    eeadrs,w,a      ;rewriting one address with the same value
        call    Rdee            ;read EEPROM data
        movf    eedatb,w,a      ;compare data in EEPROM
        xorwf   swrtmp,w,a      ;with data to write
        btfsc   status,Z,a
        return
        movff   swrtmp,eedatb
        movf    eeadrs,w,a
        call    Wree            ;write to EEPROM
        return
;
Sndsb:  bcf     pir1,SSPIF,a    ;this subroutine generates I2C start bit
        bsf     sspcon2,SEN,a   ;generate Start condition
Sndsb2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Sndsb2
        return
;
Sndpb:  bcf     pir1,SSPIF,a    ;this subroutine generates I2C stop bit
        bsf     sspcon2,PEN,a   ;generate Stop condition
Sndpb2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Sndpb2
        return
;
Sndrb:  bcf     pir1,SSPIF,a    ;this subroutine generates I2C restart bit
        bsf     sspcon2,RSEN,a  ;generate Restart condition
Sndrb2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Sndrb2
        return
;
Output: bcf     pir1,SSPIF,a    ;this subroutine serial sends Wreg
        movwf   sspbuf,a
Outpt2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Outpt2
        return
;
Input:  bcf     pir1,SSPIF,a    ;this subroutine serial reads to Wreg
        bsf     sspcon2,RCEN,a  ;initiate reception of byte
Input2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Input2
        movff   sspbuf,intmp
        bcf     pir1,SSPIF,a    ;clear SSP interrupt flag
        bsf     sspcon2,ACKEN,a ;generate ACK/NO_ACK bit
Input3: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Input3
        movf    intmp,w,a
        return
;
Rdee:   movwf   eeadrs,a        ;read EEPROM subroutine
Rdee2:  call    Sndsb           ;start bit
        movlw   B'10100000'     ;control byte - write
        call    Output          ;acknowledge pooling:
        btfss   sspcon2,ACKSTAT,a
        bra     Rdee3           ;ACK bit was sent
        call    Sndpb           ;NO_ACK, stop bit
        nop
        goto    Rdee2
Rdee3:  movf    eeadrs,w,a
        call    Output          ;write address to EEPROM
        call    Sndrb           ;restart bit
        movlw   B'10100001'     ;control byte - read
        call    Output
        bsf     sspcon2,ACKDT,a ;select NO_ACK bit
        call    Input           ;read data
        movwf   eedatb,a
        call    Sndpb           ;stop bit
        return
;
Wree:   movwf   eeadrs,a        ;write EEPROM subroutine
Wree2:  call    Sndsb           ;start bit
        movlw   B'10100000'     ;control byte - write
        call    Output          ;acknowledge pooling:
        btfss   sspcon2,ACKSTAT,a
        bra     Wree3           ;ACK bit was sent
        call    Sndpb           ;stop bit
        nop
        goto    Wree2
Wree3:  movf    eeadrs,w,a
        call    Output          ;write address to EEPROM
        movf    eedatb,w,a
        call    Output          ;write data to EEPROM
        call    Sndpb           ;stop bit
        return
;**************************************************************************
; end of PVK40 Example : Serial EEPROM 24xx (HW)
        end
