;**************************************************************************
;
;                       PVK40 Example : Serial EEPROM 24xx (SW)
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board :
; 1) Insert 24LC01B into 24xx socket
; 2) Switch off all DIP switches except:
; 3) Switch on OSC 3.276M on S9
; 4) Switch on B0,1,2,4 DISP on S11
; 5) Switch on B3 LED on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
intcnt  equ     0x20            ;interrupt counting
cnt1    equ     0x21            ;used in Wait subroutine
dig1    equ     0x22            ;digit '1'
dig2    equ     0x23            ;digit '10'
dig3    equ     0x24            ;digit '100'
dig4    equ     0x25            ;digit '1000'
digtmp1 equ     0x26            ;store dig1 value
digtmp2 equ     0x27            ;store dig2 value
digtmp3 equ     0x28            ;store dig2 value
eedatb  equ     0x29            ;EEPROM data write/read
intmp   equ     0x2A            ;used in Input subroutine
eeadrs  equ     0x2B            ;used in Wree and Rdee subroutines
swrtmp  equ     0x2C            ;used in Swree subroutine
buffer  equ     0x2D            ;used in Output and Input subroutines
bufcnt  equ     0x2E            ;used in Output and Input subroutines
bckwreg equ     0x70            ;working register backup during interrupt
bcksta  equ     0x71            ;status register backup during interrupt
; Bits definition :
#define SDA     portc,4         ;I2C data
#define SCL     portc,3         ;I2C clock
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
#define LED     portb,3         ;LED
#define PB0     portd,0         ;pushbutton 0
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        swapf   status,w        ;save status during interrupt
        movwf   bcksta
        bcf     status,RP0      ;bank 0
        bcf     intcon,T0IF     ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_D           ;then return
;
        movlw   0x0B            ;else reinitialise intcnt register
        movwf   intcnt
        incf    dig1,f          ;increment dig1
        movlw   0x0A
        subwf   dig1,w          ;if dig1 > 9 
        btfss   status,C
        goto    Int_A
        clrf    dig1            ;then dig1 = 0
        incf    dig2,f          ;and increment dig2
Int_A:  movlw   0x0A
        subwf   dig2,w          ;if dig2 > 9 
        btfss   status,C
        goto    Int_B
        clrf    dig2            ;then dig2 = 0
        incf    dig3,f          ;and increment dig3
Int_B:  movlw   0x0A
        subwf   dig3,w          ;if dig3 > 9 
        btfss   status,C
        goto    Int_C
        clrf    dig3            ;then dig3 = 0
        incf    dig4,f          ;and increment dig4
Int_C:  movlw   0x0A
        subwf   dig4,w          ;if dig4 > 9 then dig4 = 0
        btfsc   status,C
        clrf    dig4
Int_D:  swapf   bcksta,w        ;restore status register
        movwf   status
        swapf   bckwreg,f
        swapf   bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   B'11110111'     ;clear RB3 pin latch
        movwf   portb
        clrf    portc           ;clear portc latch
        movlw   0xFF            ;set portd latch
        movwf   portd
        bsf     status,RP0      ;bank 1
        movlw   B'11100000'     ;pins RB0,1,2,3,4 are outputs
        movwf   trisb
        clrf    trisd           ;portd pins are outputs
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   option_reg
        bcf     status,RP0      ;bank 0
;
        movlw   0x00            ;read address 00
        call    Rdee            ;from EEPROM
        movf    eedatb,w
        andlw   0x0F
        movwf   dig4            ;save to dig4
        movlw   0x01            ;read address 01
        call    Rdee            ;from EEPROM
        movf    eedatb,w
        andlw   0x0F
        movwf   dig3            ;save to dig3
        movlw   0x02            ;read address 02
        call    Rdee            ;from EEPROM
        movf    eedatb,w
        andlw   0x0F
        movwf   dig2            ;save to dig2
        movlw   0x03            ;read address 03
        call    Rdee            ;from EEPROM
        movf    eedatb,w
        andlw   0x0F
        movwf   dig1            ;save to dig1
;
        movlw   0x0B            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
        movlw   0x88            ;0x88 = trisd address
        movwf   fsr
;
Main:   movf    dig1,w
        call    Table
        movwf   portd
        bcf     DCOM1           ;turn on display 1
        call    Wait
        bsf     DCOM1           ;turn off display 1
        movf    dig2,w
        call    Table
        movwf   portd
        bcf     DCOM2           ;turn on display 2
        call    Wait
        bsf     DCOM2           ;turn off display 2
        movf    dig3,w
        call    Table
        movwf   portd
        bcf     DCOM3           ;turn on display 3
        call    Wait
        bsf     DCOM3           ;turn off display 3
        movf    dig4,w
        call    Table
        movwf   portd
        bcf     DCOM4           ;turn on display 4
        call    Wait
        bsf     DCOM4           ;turn off display 4
;
        bsf     indf,0          ;RD0 is input
        call    Wait            ;delay to stable inputs
        btfss   PB0             ;if pushbutton is pressed
        bsf     LED             ;turn on the LED
        btfsc   PB0
        bcf     LED
        bcf     indf,0          ;RD0 is output
;
        btfss   LED             ;if the LED is on
        goto    Main
        bcf     intcon,GIE      ;disable interrupt
        movf    dig1,w          ;save dig1-dig4 state
        movwf   digtmp1
        movf    dig2,w
        movwf   digtmp2
        movf    dig3,w
        movwf   digtmp3
        movf    dig4,w
        movwf   eedatb
        bsf     intcon,GIE      ;enable interrupt
        movlw   0x00
        call    Swree           ;write to EEPROM
        movf    digtmp3,w
        movwf   eedatb
        movlw   0x01
        call    Swree           ;write to EEPROM
        movf    digtmp2,w
        movwf   eedatb
        movlw   0x02
        call    Swree           ;write to EEPROM
        movf    digtmp1,w
        movwf   eedatb
        movlw   0x03
        call    Swree           ;write to EEPROM
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        addwf   pcl,f
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Setda:  bsf     status,RP0      ;sets SDA to 1
        bsf     SDA
        bcf     status,RP0      ;bank 0
        return
;
Clrda:  bsf     status,RP0      ;sets SDA to 0
        bcf     SDA
        bcf     status,RP0      ;bank 0
        return
;
Setcl:  bsf     status,RP0      ;sets SCL to 1
        bsf     SCL
        bcf     status,RP0      ;bank 0
Setcl2: btfss   SCL             ;wait for SCL goes Hi
        goto    Setcl2
        return
;
Clrcl:  bsf     status,RP0      ;sets SCL to 0
        bcf     SCL
        bcf     status,RP0      ;bank 0
        return
;
Swree:  movwf   eeadrs          ;this subroutine writes to EEPROM
        movf    eedatb,w        ;use this subroutine to avoid multiply
        movwf   swrtmp          ;rewriting one address with the same value
        movf    eeadrs,w
        call    Rdee            ;read EEPROM data
        movf    eedatb,w        ;compare data in EEPROM
        xorwf   swrtmp,w        ;with data to write
        btfsc   status,Z
        return
        movf    swrtmp,w
        movwf   eedatb
        movf    eeadrs,w
        call    Wree            ;write to EEPROM
        return
;
Sndsb:  call    Setda           ;this subroutine generates I2C start bit
        nop
        call    Setcl
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Clrda           ;SDA = 0
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Clrcl           ;SCL = 0
        nop
        return
;
Sndpb:  call    Clrcl           ;this subroutine generates I2C stop bit
        nop
        call    Clrda
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Setcl           ;SCL = 1
        nop                     ;wait four microseconds
        nop
        nop
        nop
        call    Setda           ;SDA = 1
        nop
        return
;
Output: movwf   buffer          ;place data in buffer
        movlw   0x08
        movwf   bufcnt
Outpt2: btfss   buffer,7        ;SDA := buffer MSb
        call    Clrda
        btfsc   buffer,7
        call    Setda
        call    Setcl           ;SCL = 1
        nop
        call    Clrcl           ;SCL = 0
        nop
        rlf     buffer,f
        decfsz  bufcnt,f
        goto    Outpt2          ;repeat 8x
        call    Setda           ;SDA = 1 (Hi-Impedance)
        call    Setcl           ;SCL = 1
        bcf     buffer,7        ;9.clock pulse
        btfsc   SDA             ;read ACKF
        bsf     buffer,7
        call    Clrcl           ;SCL = 0
        return
;
Input:  call    Setda           ;serial receive 8 bits
        movlw   0x08            ;SDA = 1 (Hi-Impedance)
        movwf   bufcnt
Inpt2:  call    Setcl           ;SCL = 1
        nop
        bcf     status,C
        btfsc   SDA             ;read data from SDA
        bsf     status,C
        rlf     intmp,f
        nop
        nop
        call    Clrcl           ;SCL = 0
        nop
        nop
        decfsz  bufcnt,f
        goto    Inpt2           ;repeat 8x
        btfss   buffer,7
        call    Clrda           ;ack will be send
        btfsc   buffer,7
        call    Setda           ;ack will not be send
        nop
        call    Setcl           ;SCL = 1 - send ACK
        nop
        nop
        call    Clrcl           ;SCL = 0
        movf    intmp,w
        return
;
Rdee:   movwf   eeadrs          ;read EEPROM subroutine
Rdee2:  call    Sndsb           ;start bit
        movlw   B'10100000'     ;control byte - write
        call    Output
        btfss   buffer,7        ;acknowledge pooling
        goto    Rdee3           ;ACK bit was sent
        call    Sndpb           ;NO_ACK, stop bit
        nop
        goto    Rdee2
Rdee3:  movf    eeadrs,w
        call    Output          ;write address to EEPROM
        call    Sndsb           ;start bit
        movlw   B'10100001'     ;control byte - read
        call    Output
        bsf     buffer,7        ;select NO_ACK bit
        call    Input           ;read data
        movwf   eedatb
        call    Sndpb           ;stop bit
        return
;
Wree:   movwf   eeadrs          ;write EEPROM subroutine
Wree2:  call    Sndsb           ;start bit
        movlw   B'10100000'     ;control byte - write
        call    Output
        btfss   buffer,7        ;acknowledge pooling
        goto    Wree3           ;ACK bit was sent
        call    Sndpb           ;NO_ACK, stop bit
        nop
        goto    Wree2
Wree3:  movf    eeadrs,w
        call    Output          ;write address to EEPROM
        movf    eedatb,w
        call    Output          ;write data to EEPROM
        call    Sndpb           ;stop bit
        return
;**************************************************************************
; end of PVK40 Example : Serial EEPROM 24xx (SW)
        end
