;**************************************************************************
;
;                           PVK40 Example : LCD
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board :
; 1) Insert MC1602 LCD module into LCD_DISPLAY_CONNECTOR
; 2) Switch off all DIP switches except:
; 3) Switch on OSC 3.276M on S9
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
cnt1    equ     0x20            ;used in T_50us subroutine
cnt2    equ     0x21            ;used in T_4ms subroutine
latchw  equ     0x22            ;save Wreg
pointer equ     0x23            ;points to characters in Table
; Bits definition :
#define LCDRS   porte,0         ;E
#define LCDRW   porte,1         ;R/W
#define LCDE    porte,2         ;RS
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  movlw   0x00
        movwf   portd           ;clear portd latch
        movwf   porte           ;clear porte latch
        bsf     status,RP0      ;bank 1
        clrf    trisd           ;portd pins are outputs
        movlw   B'00001000'     ;pins RE0,1,2 are outputs
        movwf   trise
        movlw   B'00000111'     ;porte pins as digital IO
        movwf   adcon1
        bcf     status,RP0      ;bank 0
        call    Lcdini          ;LCD initialision
;
Main:   bsf     LCDRS           ;data will be written
        bcf     LCDRW
        clrf    pointer
Main_2: call    Table
        movwf   latchw          ;make advance
        sublw   '*'
        btfsc   status,Z        ;repeat procedure until character = '*'
        goto    Loop            ;last character in table was found
        movf    latchw,W
        call    Send            ;send character from table
        incf    pointer,F       ;take next character
        goto    Main_2
;
Loop:   goto    Loop            ;never-ending loop
;--------------------------------------------------------------------------
; Subroutines :
T_50us: movlw   0x0C            ;wait 50 us
        movwf   cnt1
        decfsz  cnt1,F
        goto    $-1
        return
;
T_4ms:  movlw   0x4B            ;wait 4 ms
        movwf   cnt2
T_4ms2: call    T_50us
        decfsz  cnt2,F
        goto    T_4ms2
        return
;
Send:   movwf   portd           ;data to portd
        bsf     LCDE            ; _/
        nop                     ;write to RAM
        bcf     LCDE            ; \_
        call    T_50us          ;wait 50us
        return
;
Lcdini: bcf     LCDRS           ;we'll send commands
        bcf     LCDRW
        movlw   B'00110000'     ;comunication via 8 bits
        call    Send
        call    T_4ms
        movlw   B'00110000'
        call    Send
        call    T_50us
        movlw   B'00110000'
        call    Send
        movlw   B'00111000'     ;2-line display
        call    Send
        movlw   B'00001000'     ;display off
        call    Send
        movlw   B'00000001'     ;clear display
        call    Send
        call    T_4ms
        movlw   B'00000110'
        call    Send
        movlw   B'00001100'     ;display on
        call    Send
        return
;
Table:  movf    pointer,W       ;table of legend
        andlw   B'00001111'     ;overflow prevention
        addwf   pcl,f
        retlw   ' '
        retlw   'H'
        retlw   'e'
        retlw   'l'
        retlw   'l'
        retlw   'o'
        retlw   ','
        retlw   ' '
        retlw   'W'
        retlw   'o'
        retlw   'r'
        retlw   'l'
        retlw   'd'
        retlw   '!'
        retlw   '*'
        retlw   ' '                     ; prohibited zone
        retlw   ' '                     ; prohibited zone
        retlw   ' '                     ; prohibited zone
;**************************************************************************
; end of PVK40 Example : LCD
        end
