;**************************************************************************
;
;                           PVK40 Example : LCD
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board :
; 1) Insert MC1602 LCD module into LCD_DISPLAY_CONNECTOR
; 2) Switch off all DIP switches except:
; 3) Switch on OSC 3.276M on S9
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
cnt1    equ     0x00            ;used in T_50us subroutine
cnt2    equ     0x01            ;used in T_4ms subroutine
latchw  equ     0x02            ;save Wreg
pointer equ     0x03            ;points to characters in Table
; Bits definition :
#define LCDRS   porte,0         ;E
#define LCDRW   porte,1         ;R/W
#define LCDE    porte,2         ;RS
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  clrf    portd,a         ;clear portd latch
        clrf    porte,a         ;clear porte latch
        clrf    trisd,a         ;portd pins are outputs
        movlw   B'00001000'     ;pins RE0,1,2 are outputs
        movwf   trise,a
        movlw   B'00001111'     ;porte pins as digital IO
        movwf   adcon1,a
        call    Lcdini          ;LCD initialision
;
Main:   bsf     LCDRS,a         ;data will be written
        bcf     LCDRW,a
        clrf    pointer,a
Main_2: call    Table
        movwf   latchw,a        ;make advance
        sublw   '*'
        btfsc   status,Z,a      ;repeat procedure until character = '*'
        goto    Loop            ;last character in table was found
        movf    latchw,W,a
        call    Send            ;send character from table
        incf    pointer,f,a     ;take next character
        goto    Main_2
;
Loop:   goto    Loop            ;never-ending loop
;--------------------------------------------------------------------------
; Subroutines :
T_50us: movlw   0x0C            ;wait 50 us
        movwf   cnt1,a
        decfsz  cnt1,F,a
        goto    $-2
        return
;
T_4ms:  movlw   0x4B            ;wait 4 ms
        movwf   cnt2,a
T_4ms2: call    T_50us
        decfsz  cnt2,F,a
        goto    T_4ms2
        return
;
Send:   movwf   portd,a         ;data to portd
        bsf     LCDE,a          ; _/
        nop                     ;write to RAM
        bcf     LCDE,a          ; \_
        call    T_50us          ;wait 50us
        return
;
Lcdini: bcf     LCDRS,a         ;we'll send commands
        bcf     LCDRW,a
        movlw   B'00110000'     ;comunication via 8 bits
        call    Send
        call    T_4ms
        movlw   B'00110000'
        call    Send
        call    T_50us
        movlw   B'00110000'
        call    Send
        movlw   B'00111000'     ;2-line display
        call    Send
        movlw   B'00001000'     ;display off
        call    Send
        movlw   B'00000001'     ;clear display
        call    Send
        call    T_4ms
        movlw   B'00000110'
        call    Send
        movlw   B'00001100'     ;display on
        call    Send
        return
;
Table:  rlncf   pointer,w,a     ;table of legend
        andlw   B'00011110'     ;overflow prevention
        addwf   pcl,f,a         ;wreg * 2 -> pcl
        retlw   ' '
        retlw   'H'
        retlw   'e'
        retlw   'l'
        retlw   'l'
        retlw   'o'
        retlw   ','
        retlw   ' '
        retlw   'W'
        retlw   'o'
        retlw   'r'
        retlw   'l'
        retlw   'd'
        retlw   '!'
        retlw   '*'
        retlw   ' '                     ; prohibited zone
        retlw   ' '                     ; prohibited zone
        retlw   ' '                     ; prohibited zone
;**************************************************************************
; end of PVK40 Example : LCD
        end
