;**************************************************************************
;
;                           PVK40 Example : Alarm clock
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on B0,1,2,4 DISP on S11
; 4) Switch on B3 LED on S11
; 5) Switch on C2 BUZZER on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
cnt1    equ     0x20            ;used in Wait subroutine
cnt2    equ     0x21            ;used in Wait subroutine
bpcnt   equ     0x22            ;beep counting
discyc  equ     0x23            ;display cycle
btns    equ     0x24            ;pushbuttons state, 0 - 4
ontime  equ     0x25            ;bit 0 = alarm flag
thrsh   equ     0x26            ;real time, 10 * hours, 0 - 2
thrsl   equ     0x27            ;real time, hours, 0 - 9 / 0 - 3
tminh   equ     0x28            ;real time, 10 * minutes 0 - 5
tminl   equ     0x29            ;real time, minutes 0 - 9
tsec    equ     0x2A            ;real time, 1/2 seconds, 0 - 119
tmsec   equ     0x2B            ;real time, 1/400 seconds, 0 - 199
ahrsh   equ     0x2C            ;alarm time, 10 * hours, 0 - 2
ahrsl   equ     0x2D            ;alarm time, hours, 0 - 9 / 0 - 3
aminh   equ     0x2E            ;alarm time, 10 * minutes, 0 - 5
aminl   equ     0x2F            ;alarm time, minutes, 0 - 9
bcksta  equ     0x70            ;status register backup during interrupt
bckwreg equ     0x71            ;working register backup during interrupt
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
#define LED     portb,3         ;LED is on the RB3 pin
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;--------------------------------------------------------------------------
        org     0x04            ;interrupt vector
Interr: movwf   bckwreg         ;save wreg during interrupt
        swapf   status,w        ;save status during interrupt
        movwf   bcksta
        bcf     status,RP0      ;bank 0
        bcf     intcon,T0IF     ;clear flag
;
        incf    tmsec,F         ;increment time
        movlw   0xC8            ;1/400 second
        subwf   tmsec,W
        movlw   0x00
        btfsc   status,Z
        movwf   tmsec
        btfsc   status,Z
        incf    tsec,F
        movlw   0x78            ;1/2 second
        subwf   tsec,W
        movlw   0x00
        btfsc   status,Z
        movwf   tsec
        btfsc   status,Z
        incf    tminl,F
        movlw   0x0A            ;minutes
        subwf   tminl,W
        movlw   0x00
        btfsc   status,Z
        movwf   tminl
        btfsc   status,Z
        incf    tminh,F
        movlw   0x06            ;10 * minutes
        subwf   tminh,W
        movlw   0x00
        btfsc   status,Z
        movwf   tminh
        btfsc   status,Z
        incf    thrsl,F
        movlw   0x0A            ;hours
        btfsc   thrsh,1         ;if thrsh = 2 then thrsl = 0 .. 3
        movlw   0x04
        subwf   thrsl,W
        movlw   0x00
        btfsc   status,Z
        movwf   thrsl
        btfsc   status,Z
        incf    thrsh,F
        movlw   0x03            ;10 * hours
        subwf   thrsh,W
        btfsc   status,Z
        clrf    thrsh
;
        movf    tmsec,F         ;test if real time = alarm time
        btfss   status,Z
        goto    Tfd
        movf    tsec,F          ;tmsec = 0
        btfss   status,Z
        goto    Tfd
        movf    tminl,W         ;tsec = 0
        xorwf   aminl,W         ;test if tminl = aminl
        btfss   status,Z
        goto    Tfd
        movf    tminh,W         ;test if tminh = aminh
        xorwf   aminh,W
        btfss   status,Z
        goto    Tfd
        movf    thrsl,W         ;test if thrsl = ahrsl
        xorwf   ahrsl,W
        btfss   status,Z
        goto    Tfd
        movf    thrsh,W         ;test if thrsh = ahrsh
        xorwf   ahrsh,W
        btfss   status,Z
        goto    Tfd
        bsf     ontime,0        ;if real_time=alarm_time then set flag
;
Tfd:    bsf     DCOM1           ;turn off display 1
        bsf     DCOM2           ;turn off display 2
        bsf     DCOM3           ;turn off display 3
        bsf     DCOM4           ;turn off display 4
;
        incf    discyc,F        ;increment discyc
        movlw   0x05
        subwf   discyc,W
        btfsc   status,Z
        clrf    discyc
        movf    discyc,W
        andlw   0x07
        addwf   pcl,f
        goto    Cyc0
        goto    Cyc1
        goto    Cyc2
        goto    Cyc3
        goto    Cyc4
        goto    Start           ;error -> reset
        goto    Start           ;error -> reset
        goto    Start           ;error -> reset
;
Cyc0:   movf    portd,W         ;read portd
        movwf   btns
        movf    thrsh,W         ;real time / preset time
        btfsc   LED
        movf    ahrsh,W
        call    Table
        movwf   portd
        clrf    indf            ;portd pins are outputs
        bcf     DCOM4           ;turn on display 4
        goto    Intret
;
Cyc1:   movf    thrsl,W         ;real time / preset time
        btfsc   LED
        movf    ahrsl,W
        call    Table
        btfss   tsec,0          ;blinking dot
        andlw   B'01111111'
        btfsc   LED
        andlw   B'01111111'
        movwf   portd
        bcf     DCOM3           ;turn on display 3
        goto    Intret
;
Cyc2:   movf    tminh,W         ;real time / preset time
        btfsc   LED
        movf    aminh,W
        call    Table
        movwf   portd
        bcf     DCOM2           ;turn on display 2
        goto    Intret
;
Cyc3:   movf    tminl,W         ;real time / preset time
        btfsc   LED
        movf    aminl,W
        call    Table
        movwf   portd
        bcf     DCOM1           ;turn on display 1
        goto    Intret
;
Cyc4:   movlw   0xFF
        movwf   indf            ;portd pins are inputs
;
Intret: swapf   bcksta,w        ;restore status register
        movwf   status
        swapf   bckwreg,f
        swapf   bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        addwf   pcl,f
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0x60            ;this subroutine wait 74021 cycles
        movwf   cnt1            ;(90ms)
Wait_B: clrf    cnt2
Wait_C: decfsz  cnt2,f          ;decrement cnt2
        goto    Wait_C
        decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;--------------------------------------------------------------------------
Start:  movlw   0xFF
        movwf   portd           ;set portd
        movlw   B'00010111'
        movwf   portb           ;set RB0,1,2,4
        clrf    ccpr1h          ;PWM duty cycle = 0%
        clrf    ccpr1l
        movlw   B'00001111'     ;CCP1 mode = PWM
        movwf   ccp1con
        clrf    tmr2
        movlw   B'00000100'     ;timer2 on, prescaler 1:1
        movwf   t2con
        bsf     status,RP0      ;bank 1
        movlw   B'11100000'     ;pins RB0,1,2,3,4 are outputs
        movwf   trisb
        movlw   B'11111011'     ;pin RC2 is output
        movwf   trisc
        movlw   B'11111111'     ;PWM period
        movwf   pr2
        movlw   B'11010010'     ;prescaler assigned to TMR0
        movwf   option_reg      ;TMR0 overflow period = 2.5ms
        bcf     status,RP0      ;bank 0
        movlw   0x88            ;0x88 = trisd address
        movwf   fsr
;
        clrf    thrsh           ;start time 00:00:00.000
        clrf    thrsl
        clrf    tminh
        clrf    tminl
        clrf    tsec
        clrf    tmsec
        movlw   0x01            ;alarm time 12:00
        movwf   ahrsh
        movlw   0x02
        movwf   ahrsl
        clrf    aminh
        clrf    aminl
        clrf    discyc
        movlw   0xFF
        movwf   btns
        clrf    ontime
;
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
;
Main:   btfss   btns,0          ;test pushbutton 0
        goto    Btn0
        btfss   btns,4          ;test pushbutton 4
        goto    Btn4
        btfss   btns,5          ;test pushbutton 5
        goto    Btn5
        btfss   btns,6          ;test pushbutton 6
        goto    Btn6
        btfss   btns,7          ;test pushbutton 7
        goto    Btn7
        btfss   ontime,0        ;test ontime flag
        goto    Main            ;closes the loop
;
        movlw   0x4B            ;Alarm
        movwf   bpcnt
        bcf     ontime,0
Alarm:  movlw   0x80            ;PWM on
        movwf   ccpr1l
        call    Wait
        clrf    ccpr1l          ;PWM off
        call    Wait
        call    Wait
        incf    btns,W          ;test pushbuttons
        btfss   status,Z
        goto    Alarm2
        decfsz  bpcnt,F
        goto    Alarm
Alarm2: incf    btns,W          ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Alarm2
        goto    Main
;--------------------------------------------------------------------------
Btn0:   btfss   LED             ;change LED state
        goto    Btn0_A
        bcf     LED
        goto    Btn0_B
Btn0_A: bsf     LED
Btn0_B: btfss   btns,0          ;wait for release
        goto    Btn0_B
        goto    Main
;
Btn4:   bcf     intcon,GIE      ;disable interrupt
        btfsc   LED             ;real time / alarm time
        goto    Btn4_A
        incf    thrsl,F         ;increment real time
        movlw   0x0A            ;hours
        btfsc   thrsh,1         ;if thrsh = 2 then thrsl = 0 .. 3
        movlw   0x04
        subwf   thrsl,W
        movlw   0x00
        btfsc   status,Z
        movwf   thrsl
        btfsc   status,Z
        incf    thrsh,F
        movlw   0x03            ;10 * hours
        subwf   thrsh,W
        btfsc   status,Z
        clrf    thrsh
        goto    Btn4_B
;
Btn4_A: incf    ahrsl,F         ;increment alarm time
        movlw   0x0A            ;hours
        btfsc   ahrsh,1         ;if ahrsh = 2 then ahrsl = 0 .. 3
        movlw   0x04
        subwf   ahrsl,W
        movlw   0x00
        btfsc   status,Z
        movwf   ahrsl
        btfsc   status,Z
        incf    ahrsh,F
        movlw   0x03            ;10 * hours
        subwf   ahrsh,W
        btfsc   status,Z
        clrf    ahrsh
Btn4_B: bsf     intcon,GIE      ;enable interrupt
        incf    btns,W          ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Btn4_B
        goto    Main
;
Btn5:   bcf     intcon,GIE      ;disable interrupt
        btfsc   LED             ;real time / alarm time
        goto    Btn5_A
        movlw   0x01            ;decrement real time
        subwf   thrsl,W         ;hours
        btfss   status,C
        movlw   0x09
        movwf   thrsl
        movlw   0x01            ;10 * hours
        btfsc   status,C
        movlw   0x00
        subwf   thrsh,F
        btfsc   status,C
        goto    Btn5_B
        movlw   0x02
        movwf   thrsh
        movlw   0x03
        movwf   thrsl
        goto    Btn5_B
;
Btn5_A: movlw   0x01            ;decrement alarm time
        subwf   ahrsl,W         ;hours
        btfss   status,C
        movlw   0x09
        movwf   ahrsl
        movlw   0x01            ;10 * hours
        btfsc   status,C
        movlw   0x00
        subwf   ahrsh,F
        btfsc   status,C
        goto    Btn5_B
        movlw   0x02
        movwf   ahrsh
        movlw   0x03
        movwf   ahrsl
Btn5_B: bsf     intcon,GIE      ;enable interrupt
        incf    btns,W          ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Btn5_B
        goto    Main
;
Btn6:   bcf     intcon,GIE      ;disable interrupt
        btfsc   LED             ;real time / alarm time
        goto    Btn6_A
        incf    tminl,F         ;increment real time
        movlw   0x0A            ;minutes
        subwf   tminl,W
        movlw   0x00
        btfsc   status,Z
        movwf   tminl
        btfsc   status,Z
        incf    tminh,F
        movlw   0x06            ;10 * minutes
        subwf   tminh,W
        movlw   0x00
        btfsc   status,Z
        movwf   tminh
        goto    Btn6_B
;
Btn6_A: incf    aminl,F         ;increment alarm time
        movlw   0x0A            ;minutes
        subwf   aminl,W
        movlw   0x00
        btfsc   status,Z
        movwf   aminl
        btfsc   status,Z
        incf    aminh,F
        movlw   0x06            ;10 * minutes
        subwf   aminh,W
        movlw   0x00
        btfsc   status,Z
        movwf   aminh
Btn6_B: bsf     intcon,GIE      ;enable interrupt
        incf    btns,W          ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Btn6_B
        goto    Main
;
Btn7:   bcf     intcon,GIE      ;disable interrupt
        btfsc   LED             ;real time / alarm time
        goto    Btn7_A
        movlw   0x01            ;decrement real time
        subwf   tminl,W         ;minutes
        btfss   status,C
        movlw   0x09
        movwf   tminl
        movlw   0x01            ;10 * minutes
        btfsc   status,C
        movlw   0x00
        subwf   tminh,W
        btfss   status,C
        movlw   0x05
        movwf   tminh
        goto    Btn7_B
;
Btn7_A: movlw   0x01            ;decrement alarm time
        subwf   aminl,W         ;minutes
        btfss   status,C
        movlw   0x09
        movwf   aminl
        movlw   0x01            ;10 * minutes
        btfsc   status,C
        movlw   0x00
        subwf   aminh,W
        btfss   status,C
        movlw   0x05
        movwf   aminh
Btn7_B: bsf     intcon,GIE      ;enable interrupt
        incf    btns,W          ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Btn7_B
        goto    Main
;**************************************************************************
; end of PVK40 Example : Alarm clock
        end
