;**************************************************************************
;
;                           PVK40 Example : Security alarm
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on TEMP on S9
; 4) Switch on PHOTO on S9
; 5) Switch on MIC on S9
; 6) Switch on B3 LED on S11
; 7) Switch on C2 BUZZER on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
; EEPROM data :
        org     0xF00000
        de      0x00,0x01,0x02,0x03      ;code (0123)
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
cnt1    equ     0x00            ;used in Wait subroutine
cnt2    equ     0x01            ;used in Wait subroutine
intcyc  equ     0x02            ;interrupt cycle
modcnt  equ     0x03            ;used in Mod subroutine
scnt    equ     0x04            ;used in Siren subroutine
bcnt    equ     0x05            ;LED blink counting
codcnt  equ     0x06            ;code digit counting
codd1   equ     0x07            ;code, digit 1
codd2   equ     0x08            ;code, digit 2
codd3   equ     0x09            ;code, digit 3
codd4   equ     0x0A            ;code, digit 4
ecodd1  equ     0x0B            ;entered code, digit 1
ecodd2  equ     0x0C            ;entered code, digit 2
ecodd3  equ     0x0D            ;entered code, digit 3
ecodd4  equ     0x0E            ;entered code, digit 4
templ   equ     0x0F            ;temperature, LSB
temph   equ     0x10            ;temperature, MSB
photol  equ     0x11            ;photosensor value, LSB
photoh  equ     0x12            ;photosensor value, MSB
micl    equ     0x13            ;sound level, LSB
mich    equ     0x14            ;sound level, MSB
atempl  equ     0x15            ;actual temperature, LSB
atemph  equ     0x16            ;actual temperature, MSB
aphotol equ     0x17            ;actual photosensor value, LSB
aphotoh equ     0x18            ;actual photosensor value, MSB
amicl   equ     0x19            ;actual sound level, LSB
amich   equ     0x1A            ;actual sound level, MSB
intcyct equ     0x1B            ;temporary interrupt cycle
peak1   equ     0x1C            ;microphone maximum, LSB
peak2   equ     0x1D            ;microphone maximum, MSB
down1   equ     0x1E            ;microphone minimum, LSB
down2   equ     0x1F            ;microphone minimum, MSB
cmptol  equ     0x20            ;used in Cmp subroutine, tolerance
cmptl   equ     0x21            ;used in Cmp subroutine
cmpth   equ     0x22            ;used in Cmp subroutine
cmpres  equ     0x23            ;Comp subroutine result 
bckfsrl equ     0x24            ;FSR0 register backup during interrupt
bckfsrh equ     0x25            ;FSR0 register backup during interrupt
ilbl    equ     0x26            ;LED blink counting
; Bits definition :
#define LED     portb,3         ;LED is on the RB3 pin
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;--------------------------------------------------------------------------
        org     0x08            ;interrupt vector
Inter:  bcf     intcon,T0IF,a   ;clear flag
        movff   fsr0l,bckfsrl
        movff   fsr0h,bckfsrh
;
        movf    intcyc,W,a      ;test intcyc
        andlw   B'01111111'
        movwf   intcyct,a
        sublw   0x05
        movlw   0x06
        btfss   status,C,a      ;if intcyct > 5
        movwf   intcyct,a       ;then intcyct = 6
        movf    intcyc,W,a      ;if intcyc = 127
        andlw   B'01111111'
        xorlw   B'01111111'
        movlw   0x07
        btfsc   status,Z,a      ;then intcyct = 7
        movwf   intcyct,a
;
        rlncf   intcyct,w,a     ;skip to corresponding Cycx
        addwf   pcl,f,a
        bra     Cyc0
        bra     Cyc1
        bra     Cyc2
        bra     Cyc3
        bra     Cyc4
        bra     Cyc5
        bra     Cyc6
        bra     Cyc7
;
Cyc0:   movlw   B'00000001'     ;Cycle 0 - select temperature sensor
        movwf   adcon0,a
        goto    Cycfin
;
Cyc1:   bsf     adcon0,GO,a     ;Cycle 1 - start conversion
        goto    Cycfin
;
Cyc2:   btfsc   adcon0,GO,a     ;Cycle 2 - save temperature
        bra     Cyc2            ;and select photosensor
        movff   adresh,atemph
        movff   adresl,atempl
        movlw   B'00000101'
        movwf   adcon0,a
        goto    Cycfin
;
Cyc3:   bsf     adcon0,GO,a     ;Cycle 3 - start conversion
        goto    Cycfin
;
Cyc4:   btfsc   adcon0,GO,a     ;Cycle 4 - save photosensor value
        bra     Cyc4            ;and select microphone
        movff   adresh,aphotoh
        movff   adresl,aphotol
        movlw   B'00010001'
        movwf   adcon0,a
        goto    Cycfin
;
Cyc5:   bsf     adcon0,GO,a     ;Cycle 5 - start conversion
        clrf    peak1,a
        clrf    peak2,a
        setf    down1,a
        setf    down2,a
        goto    Cycfin
;
Cyc6:   btfsc   adcon0,GO,a     ;Cycle 6..126 - save mic. min/max value
        bra     Cyc6            ;and start new conversion
        movlw   adresl          ;FSR0 points to adres register
        movwf   fsr0l,a
        movlw   0x0F
        movwf   fsr0h,a
        movlw   peak1           ;FSR1 points to peak register
        movwf   fsr1l,a
        clrf    fsr1h,a
        call    Comp            ;compare
        btfss   cmpres,0,a      ;if adresh:adresl > peak2:peak1
        bra     Cyc6_A
        movff   adresh,peak2    ;save adresh:adresl to peak2:peak1
        movff   adresl,peak1
        bra     Cyc6_B
Cyc6_A: movlw   down1           ;FSR1 points to down register
        movwf   fsr1l,a
        call    Comp            ;compare
        btfss   cmpres,7,a      ;if adresh:adresl < down2:down1
        bra     Cyc6_B
        movff   adresh,down2    ;save adresh:adresl to down2:down1
        movff   adresl,down1
Cyc6_B: bsf     adcon0,GO,a
        goto    Cycfin
;
Cyc7:   movf    down1,W,a       ;Cycle 7 - save sound level
        subwf   peak1,W,a       ;peak2:peak1 - down2:down1 to amich:l
        movwf   amicl,a
        movf    down2,W,a
        subwfb  peak2,W,a
        movwf   amich,a
        goto    Cycfin
;
Cycfin: incf    intcyc,F,a
        movlw   0x80
        subwf   intcyc,W,a      ;if intcyc = 128
        btfsc   status,Z,a
        goto    Svcon           ;then save primary values
        tstfsz  intcyc,a        ;if intcyc <> 0
        goto    Intret          ;then return
        movlw   0x80            ;if intcyc = 0 then intcyc = 128
        movwf   intcyc,a
;
        tstfsz  ilbl,a          ;test if ilbl = 0
        goto    Nlbl
        clrf    fsr0h,a
        clrf    fsr2h,a
        movlw   0x01
        movwf   cmptol,a        ;tolerance
        movlw   templ           ;temp register address
        movwf   fsr0l,a
        movlw   atempl          ;atemp register address
        movwf   fsr2l,a
        call    Cmpt
        tstfsz  wreg,a          ;test result
        bra     Slbl
        movlw   0x20
        movwf   cmptol,a        ;tolerance
        movlw   photol          ;photo register address
        movwf   fsr0l,a
        movlw   aphotol         ;aphoto register address
        movwf   fsr2l,a
        call    Cmpt
        tstfsz  wreg,a          ;test result
        bra     Slbl
        movlw   0x35
        movwf   cmptol,a        ;tolerance
        movlw   micl            ;mic register address
        movwf   fsr0l,a
        movlw   amicl           ;amic register address
        movwf   fsr2l,a
        call    Cmpt
        tstfsz  wreg,a          ;test result
        bra     Slbl
        goto    Intret          ;values OK, return
;
Slbl:   movlw   0x01            ;event on sensor
        movwf   ilbl,a          ;start LED blinking
        goto    Intret
;
Nlbl:   incf    ilbl,F,a        ;increment time
        btg     LED,a           ;invert LED state
        movlw   0x20            ;max. time
        cpfsgt  ilbl,a
        bra     Intret
        bsf     LED,a
        call    Siren           ;time overflow, start siren
        goto    Enin
;
Svcon:  movff   atempl,templ    ;save primary values
        movff   atemph,temph
        movff   aphotol,photol
        movff   aphotoh,photoh
        movff   amicl,micl
        movff   amich,mich
        clrf    ilbl,a          ;clear ilbl = no event
;
Intret: movff   bckfsrl,fsr0l
        movff   bckfsrh,fsr0h 
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
; Subroutines :
Rdee:   movwf   eeadr,a         ;read from internal EEPROM
        bcf     eecon1,EEPGD,a  ;point to Data memory
        bsf     eecon1,RD,a     ;start read operation
        movf    eedata,W,a      ;W = EEDATA
        return
;
Mod:    andlw   0x07            ;modify 0-7 to pushbutton code
        addlw   0x01
        movwf   modcnt,a
        movlw   0x7F
Mod2:   rlncf   wreg,f,a        ;rotate
        decfsz  modcnt,f,a
        bra     Mod2
        return
;
Beep:   movlw   B'01000000'     ;PWM period
        movwf   pr2,a
        movlw   0x02            ;PWM on
        movwf   ccpr1l,a
        call    Wait
        clrf    ccpr1l,a        ;PWM off
        return
;
Siren:  movlw   0x20            ;Siren
        movwf   scnt,a
Siren2: movlw   B'01000000'     ;PWM period
        movwf   pr2,a
        movlw   B'00100000'     ;PWM on
        movwf   ccpr1l,a
        call    Wait
        call    Wait
        call    Wait
        movlw   B'10000000'     ;PWM period
        movwf   pr2,a
        movlw   B'01000000'
        movwf   ccpr1l,a
        call    Wait
        call    Wait
        call    Wait
        decfsz  scnt,F,a
        goto    Siren2
        clrf    ccpr1l,a        ;PWM off
        return
;
Wait:   movlw   0x60            ;this subroutine wait 74021 cycles
        movwf   cnt1,a          ;(90ms)
Wait_B: clrf    cnt2,a
Wait_C: decfsz  cnt2,f,a        ;decrement cnt2
        goto    Wait_C
        decfsz  cnt1,f,a        ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Keyin:  movlw   ecodd1          ;read 4 - digit code
        movwf   fsr0l,a
        clrf    fsr0h,a
        clrf    codcnt,a
Keyin2: incf    portd,W,a       ;test pushbuttons
        bz      Keyin2          ;wait for press pushbutton
        addlw   0xFF            ;subtract 1
        movwf   postinc0,a      ;save pushbuttons state
        call    Beep
Keyin3: incf    portd,W,a       ;test pushbuttons
        bnz     Keyin3          ;wait for release
        call    Wait
        incf    codcnt,F,a
        btfss   codcnt,2,a
        bra     Keyin2          ;repeat 4x
;
        movf    codd1,W,a       ;compare codd4:1 with ecodd4:1
        cpfseq  ecodd1,a
        retlw   0xFF
        movf    codd2,W,a
        cpfseq  ecodd2,a
        retlw   0xFF
        movf    codd3,W,a
        cpfseq  ecodd3,a
        retlw   0xFF
        movf    codd4,W,a
        cpfseq  ecodd4,a
        retlw   0xFF
        retlw   0x00            ;code OK
;
Comp:   movf    preinc0,w,a     ;compare two 16-bit register A,B
        cpfsgt  preinc1,a       ;A = *(FSR0,FSR0+1), B = *(FSR1,FSR1+1)
        bra     Comp_B          ;A < B => cmpres = B'11100000'
        movf    postdec0,f,a    ;A = B => cmpres = B'00011000'
        movf    postdec1,f,a    ;A > B => cmpres = B'00000111'
Comp_A: movlw   B'11100000'
        movwf   cmpres,a        ;A < B
        return
Comp_B: movf    postdec0,w,a
        cpfslt  postdec1,a
        bra     Comp_D
Comp_C: movlw   B'00000111'     ;A > B
        movwf   cmpres,a
        return
Comp_D: movf    indf0,w,a
        cpfseq  indf1,a
        bra     Comp_E
        movlw   B'00011000'     ;A = B
        movwf   cmpres,a
        return
Comp_E: cpfsgt  indf1,a
        goto    Comp_C
        goto    Comp_A
;
Cmpt:   movlw   cmptl           ;compare 16-bit number A,B +/-cmptol
        movwf   fsr1l,a         ;A = *(FSR0,FSR0+1), B = *(FSR2,FSR2+1)
        clrf    fsr1h,a         ;A = B => return 0x00 else return 0xFF
        movff   postinc2,postinc1
        movff   postdec2,postdec1
        movf    cmptol,w,a      ;add tolerance
        addwf   postinc1,f,a
        movlw   0x00
        addwfc  postdec1,f,a
        bnc     $+6             ;if overflow,
        setf    postinc1,a      ;set INDF1
        setf    postdec1,a
        call    Comp            ;compare
        btfsc   cmpres,0,a
        retlw   0xFF            ;nonequal
;
        movff   postinc2,postinc1
        movff   postdec2,postdec1
        movf    cmptol,w,a      ;subtract tolerance
        subwf   postinc1,f,a
        movlw   0x00
        subwfb  postdec1,f,a
        bnn     $+6             ;if negative,
        clrf    postinc1,a      ;clear INDF1
        clrf    postdec1,a
        call    Comp            ;compare
        btfsc   cmpres,7,a
        retlw   0xFF            ;nonequal
        retlw   0x00            ;in tolerance
;--------------------------------------------------------------------------
Start:  clrf    portb,a         ;clear RB3
        movlw   B'11110111'     ;pin RB3 is output
        movwf   trisb,a
        movlw   B'11111011'     ;pin RC2 is output
        movwf   trisc,a
        clrf    ccpr1h,a        ;PWM duty cycle = 0%
        clrf    ccpr1l,a
        movlw   B'00001100'     ;CCP1 mode = PWM
        movwf   ccp1con,a
        clrf    tmr2,a
        movlw   B'00000101'     ;timer2 on, prescaler 1:4
        movwf   t2con,a
        movlw   B'00000001'     ;enable A/D converter module
        movwf   adcon0,a
        movlw   B'00001010'     ;pins RA0-RA5 are analog inputs
        movwf   adcon1,a
        movlw   B'10001100'     ;A/D Conversion Clock = FOSC/4
        movwf   adcon2,a        ;A/D Acquisition Time = 8 / FOSC
        movlw   B'11000001'     ;prescaler assigned to TMR0
        movwf   t0con,a         ;TMR0 overflow period = 1.25ms
        clrf    tmr0h,a
        clrf    tmr0l,a
        bcf     rcon,IPEN,a     ;disable priority interrupt
        movlw   B'00100000'     ;unmask interrupt from TMR0
        movwf   intcon,a
;
        movlw   codd4           ;read code from eeprom
        movwf   fsr0l,a
        clrf    fsr0h,a
        movlw   0x04
        movwf   codcnt,a
Codrd:  decf    codcnt,W,a
        call    Rdee            ;read eeprom
        call    Mod             ;modify to pushbutton code
        movwf   postdec0,a
        decfsz  codcnt,F,a      ;4x
        bra     Codrd
;
Inact:  bcf     LED,a           ;turn off the LED
        call    Keyin           ;read 4 - digit code
        addlw   0x01
        btfss   status,Z,a      ;test result
        bra     Blled           ;code OK
        call    Siren           ;wrong code
        bra     Inact
;
Blled:  movlw   0x40            ;LED blink
        movwf   bcnt,a
Blled2: bcf     LED,a
        call    Wait
        bsf     LED,a
        call    Wait
        decfsz  bcnt,F,a
        bra     Blled2
;
Enin:   call    Wait
        clrf    tmr0l,a
        bcf     intcon,T0IF,a   ;clear TMR0 flag
        clrf    intcyc,a        ;start with cycle 0
        bsf     intcon,GIE,a    ;enable interrupt
;
        call    Keyin           ;wait for 4 - digit code
        bcf     intcon,GIE,a    ;disable interrupt
        bsf     LED,a
        addlw   0x01
        btfss   status,Z,a      ;test result
        goto    Inact           ;code OK
        call    Siren           ;wrong code
        goto    Enin
;**************************************************************************
; end of PVK40 Example : Security alarm
        end
