;**************************************************************************
;
;                PVK40 Example : Infra-Red Receiver
;
; Note: UIR remote decoder, compatible with IRAMP/IRMAN
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none
;
; On your PVK40 board :
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on C0 IRF_RX on S11
; 4) Switch on C6 UART_TX on S11
; 5) Switch on C7 UART_RX on S11
; 6) Select COM or USB on JP4
; 7) If USB selected, insert UMS1 module into socket
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
rcdata  equ     0x20            ;received serial data
state   equ     0x21            ;bit0 : 1 = last received byte was 'I'
bcnt    equ     0x22            ;used in Sndbl,Cpy6b,Capuir subroutines
cpytmp  equ     0x23            ;used in Cpy6b subroutine
timel   equ     0x24            ;copy of tmr1l register
timeh   equ     0x25            ;copy of tmr1h register
ltmml   equ     0x26            ;used in Cpuir subroutine
ltmmh   equ     0x27            ;used in Cpuir subroutine
ltmsl   equ     0x28            ;used in Cpuir subroutine
ltmsh   equ     0x29            ;used in Cpuir subroutine
mask    equ     0x2A            ;used in Cpuir subroutine
xmask   equ     0x2B            ;used in Cpuir subroutine
bits    equ     0x2C            ;used in Cpuir subroutine
cmptol  equ     0x2D            ;used in Cmp subroutine, tolerance
cmpal   equ     0x2E            ;used in Cmp subroutine
cmpah   equ     0x2F            ;used in Cmp subroutine
;
uircd   equ     0x40            ;IR received data
; bytes 0x41 - 0x45 are accessed via INDF
puircd  equ     0x46            ;repeated IR received data
; bytes 0x47 - 0x4B are accessed via INDF
; Bits definition :
#define IRFRX   portc,0         ;IR input
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  bsf     status,RP0      ;bank 1
        movlw   B'10111111'     ;pin RC6 is output (optional, RC6 is
        movwf   trisc           ; automaticaly configured as output if
        movlw   0x14            ; UART is enabled)
        movwf   spbrg           ;9600 Bd
        movlw   B'00100110'     ;enable transmit
        movwf   txsta
        bcf     status,RP0      ;bank 0
        movlw   B'00100001'     ;enable timer1
        movwf   t1con
        movlw   B'10010000'     ;serial port enabled
        movwf   rcsta
        movf    rcreg,W         ;clear RCIF
        clrf    state
;
Main:   btfss   pir1,RCIF       ;test rx flag
        goto    Irtest
        movf    rcreg,W         ;copy rcreg to rcdat
        movwf   rcdata
        movlw   'I'             ;if received data = 'I'
        xorwf   rcdata,W
        btfsc   status,Z
        goto    Rci             ;go to Rci
        movlw   'R'             ;if received data = 'R'
        xorwf   rcdata,W
        btfsc   status,Z
        goto    Rcr             ;go to Rcr
        bcf     state,0         ;else state,0 = 0
        goto    Irtest
Rci:    bsf     state,0         ;state,0 = 1
        goto    Irtest
Rcr:    btfss   state,0         ;if state,0 = 1
        goto    Irtest
        movlw   'O'             ;send "OK" to PC
        call    Send
        movlw   'K'
        call    Send
        bcf     state,0         ;state,0 = 0
;
Irtest: btfsc   IRFRX           ;test IR input
        goto    Main
        btfsc   pir1,TMR1IF     ;if timer overflow
        bcf     state,1         ;mark repeat data as old
        call    Cpuir           ;capture UIR
;
        andlw   0xF0
        btfsc   status,Z        ;if result < 16
        goto    Rptt            ;then skip to Rptt
        bsf     state,1
        movlw   uircd
        call    Sndbl           ;send 6 bytes
        call    Cpy6b           ;copy uircd to puircd
        call    Clrtmr
        goto    Main
;
Rptt:   btfss   state,1         ;repeat?
        goto    Main            ;no
        movlw   puircd          ;yes
        call    Sndbl           ;send 6 bytes
        call    Clrtmr
        goto    Main
;--------------------------------------------------------------------------
; Subroutines :
Send:   movwf   txreg           ;send data
        bsf     status,RP0      ;bank 1
Send2:  btfss   txsta,TRMT
        goto    Send2
        bcf     status,RP0      ;bank 0
        return
;
Sndbl:  movwf   fsr             ;send 6-byte block
        movlw   0x06
        movwf   bcnt
Sndbl2: movf    indf,W
        call    Send
        incf    fsr,F
        decfsz  bcnt,F
        goto    Sndbl2
        return
;
Clrtmr: clrf    tmr1l           ;clear timer1 register
        clrf    tmr1h
        bcf     pir1,TMR1IF
        return
;
Cpy6b:  clrf    bcnt            ;copy 6 bytes from uircd:uircd+5
Cpy6b2: movlw   uircd           ;to puircd:puircd+5
        addwf   bcnt,W
        movwf   fsr
        movf    indf,W
        movwf   cpytmp
        movlw   puircd
        addwf   bcnt,W
        movwf   fsr
        movf    cpytmp,W
        movwf   indf
        incf    bcnt,F
        movlw   0x06
        xorwf   bcnt,W
        btfss   status,Z
        goto    Cpy6b2
        return
;
Cpuir:  movlw   0x06            ;capture UIR code
        movwf   bcnt
        movlw   uircd           ;clear uircd:uircd+5
        movwf   fsr
Cpuir2: clrf    indf
        incf    fsr,F
        decfsz  bcnt,F
        goto    Cpuir2
;
        call    Clrtmr          ;clear timer 1
        call    Measm           ;wait for IRFRX = 1
        movf    timeh,W         ;test time
        andlw   0xFF
        btfsc   status,Z
        retlw   0x00            ;error, return 0
        call    Meass           ;wait for IRFRX = 0
        movf    timeh,W         ;test time
        andlw   0xF8
        btfss   status,Z
        retlw   0x00            ;error, return 0
;
        movlw   B'10000000'
        movwf   mask
        movlw   0x20
        movwf   cmptol
        clrf    bcnt
        clrf    bits
        clrf    ltmml
        clrf    ltmmh
        clrf    ltmsl
        clrf    ltmsh
;
Cpuir3: call    Measm           ;wait for IRFRX = 1
        movf    timeh,W         ;test time
        andlw   0xF8
        btfss   status,Z
        goto    Cpuir5          ;time-out, return
        movlw   ltmml
        call    Comp            ;compare with previous value
        andwf   mask,W
        movwf   xmask
        movlw   uircd
        addwf   bcnt,W
        movwf   fsr
        movf    indf,W
        xorwf   xmask,W
        movwf   indf
;
        movf    timel,W         ;copy timeh:l to ltmmh:l
        movwf   ltmml
        movf    timeh,W
        movwf   ltmmh
        incf    bits,F          ;increment number of bits
        bcf     status,C
        rrf     mask,F
        btfss   status,C        ;if bit C is set
        goto    Cpuir4
        bsf     mask,7          ;mask = 0x80
        incf    bcnt,F          ;increment bcnt
        movlw   0x06
        xorwf   bcnt,W          ;if bcnt = 6
        btfsc   status,Z
        clrf    bcnt            ;then bcnt = 0
;
Cpuir4: call    Meass           ;wait for IRFRX = 0
        movf    timeh,W         ;test time
        andlw   0xF8
        btfss   status,Z
        goto    Cpuir5          ;time-out, return
        movlw   ltmsl
        call    Comp            ;compare with previous value
        andwf   mask,W
        movwf   xmask
        movlw   uircd
        addwf   bcnt,W
        movwf   fsr
        movf    indf,W
        xorwf   xmask,W
        movwf   indf
;
        movf    timel,W         ;copy timeh:l to ltmsh:l
        movwf   ltmsl
        movf    timeh,W
        movwf   ltmsh
        incf    bits,F          ;increment number of bits
        bcf     status,C
        rrf     mask,F
        btfss   status,C        ;if bit C is set
        goto    Cpuir3
        bsf     mask,7          ;mask = 0x80
        incf    bcnt,F          ;increment bcnt
        movlw   0x06
        xorwf   bcnt,W          ;if bcnt = 6
        btfsc   status,Z
        clrf    bcnt            ;then bcnt = 0
        goto    Cpuir3          ;loop
;
Cpuir5: movf    bits,W
        return
;
Gettm:  bcf     t1con,TMR1ON   ;copy tmr1h:l to timeh:l
        movf    tmr1h,W
        movwf   timeh
        movf    tmr1l,W
        movwf   timel
        bsf     t1con,TMR1ON
        return
;
Measm:  call    Gettm           ;wait for IRFRX = 1
        movf    timeh,W
        andlw   0xF8
        btfss   status,Z
        goto    Measm2
        btfss   IRFRX
        goto    Measm
Measm2: call    Clrtmr
        return
;
Meass:  call    Gettm           ;wait for IRFRX = 0
        movf    timeh,W
        andlw   0xF8
        btfss   status,Z
        goto    Meass2
        btfsc   IRFRX
        goto    Meass
Meass2: call    Clrtmr
        return
;
Comp:   movwf   fsr             ;compare 16-bit number *Wreg +/-cmptol
        movf    indf,W          ;with 16-bit number timeh:l
        movwf   cmpal           ;copy indf to cmpah:l
        incf    fsr,F
        movf    indf,W
        movwf   cmpah
;
        movf    cmptol,W        ;cmpah:l = cmpah:l - tolerance
        subwf   cmpal,F
        btfsc   status,C
        goto    Comp2
        movlw   0xFF            ;subtract 1
        addwf   cmpah,F
        btfsc   status,C        ;if cmpah:l < tolerance
        goto    Comp2
        clrf    cmpal           ;then cmpah:l = 0
        clrf    cmpah
;
Comp2:  movf    cmpah,W         ;compare cmpah:l with timeh:l
        subwf   timeh,W
        btfss   status,C
        retlw   0xFF            ;nonequal
        btfss   status,Z
        goto    Comp3
        movf    cmpal,W
        subwf   timel,W
        btfss   status,C
        retlw   0xFF            ;nonequal
;
Comp3:  decf    fsr,F           ;copy indf to cmpah:l
        movf    indf,W
        movwf   cmpal
        incf    fsr,F
        movf    indf,W
        movwf   cmpah
;
        movf    cmptol,W        ;cmpah:l = cmpah:l + tolerance
        addwf   cmpal,F
        btfss   status,C
        goto    Comp4
        incf    cmpah,F
        btfss   status,Z        ;if cmpah:l + tolerance > 0xFFFF
        goto    Comp4
        movlw   0xFF            ;then cmpah:l = 0xFFFF
        movwf   cmpal
        movwf   cmpah
;
Comp4:  movf    timeh,W         ;compare cmpah:l with timeh:l
        subwf   cmpah,W
        btfss   status,C
        retlw   0xFF            ;nonequal
        btfss   status,Z
        retlw   0x00            ;in tolerance
        movf    timel,W
        subwf   cmpal,W
        btfss   status,C
        retlw   0xFF            ;nonequal
        retlw   0x00            ;in tolerance
;**************************************************************************
; end of PVK40 Example : Infra-Red Receiver
        end
