;**************************************************************************
;
;                      PVK40 Example : Sound recording
;
; Processor : PIC18F4520
; Clock : 13.1072 MHz, crystal 3.2768 MHz with x4 PLL used
;
; Serial port : 115200 Bd,8 db,parity none,1 stop bit,flow control none
;
; On your PVK40 board :
; 1) Insert 24FC515 into 24xx socket ***
; 2) Switch off all DIP switches except:
; 3) Switch on OSC 3.276M on S9
; 4) Switch on MIC on S9
; 5) Switch on B3 LED on S11
; 6) Switch on C6 UART_TX on S11
; 7) If USB selected, insert UMS1 module into socket
;
; *** Note: pin 3 (A2) must be tied to Vcc
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x06  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
; EEPROM data :
        org     0xF00000
        de      0x2C,0x52,0x49,0x46,0x46,0x24,0x00,0x01 ;wav file header
        de      0x00,0x57,0x41,0x56,0x45,0x66,0x6D,0x74 ;first byte
        de      0x20,0x10,0x00,0x00,0x00,0x01,0x00,0x01 ;is header length
        de      0x00,0x40,0x1F,0x00,0x00,0x40,0x1F,0x00
        de      0x00,0x01,0x00,0x08,0x00,0x64,0x61,0x74
        de      0x61,0x00,0x00,0x01,0x00,0x00,0x00,0x00
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
bcnt    equ     0x00            ;byte counting
intmp   equ     0x01            ;used in Input subroutine
ieaddr  equ     0x02            ;internal EEPROM address pointer
eaddrl  equ     0x03            ;I2C EEPROM address pointer, LSB
eaddrh  equ     0x04            ;I2C EEPROM address pointer, MSB
eedatb  equ     0x05            ;used in Wree and Rdee subroutines
reccon  equ     0x06            ;used during capture
;  bytes 0x80 - 0xFF are used as 2*64B buffer
;  these bytes are accessed via indirect addressing
; Bits definition :
#define RDB0    reccon,0        ;block 0 data valid
#define RDB1    reccon,1        ;block 0 data valid
#define WRT0    reccon,2        ;writing block 0
#define WRT1    reccon,3        ;writing block 1
#define LED     portb,3         ;LED is on the RB3 pin
#define PB0     portd,0         ;pushbutton 0 - record
#define PB7     portd,7         ;pushbutton 7 - send to PC
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;--------------------------------------------------------------------------
        org     0x08            ;interrupt vector
Interr: bcf     intcon,TMR0IF,a ;clear flag
        movlw   0x9C            ;TMR0 period = 125usec
        movwf   tmr0l,a
;
Adw:    btfsc   adcon0,GO,a     ;wait for A/D conversion to complete
        bra     Adw
        btfsc   WRT0,a
        goto    Nwb0            ;writing block 0
        btfsc   WRT1,a
        goto    Nwb1            ;writing block 1
        btfss   RDB0,a
        goto    Swb0            ;start write to block 0
        btfss   RDB1,a
        goto    Swb1            ;start write to block 1
        goto    Intret          ;busy -> wait
;
Nwb0:   movff   adresh,postinc1 ;copy data to buffer
        movlw   0xC0
        cpfseq  fsr1l,a         ;if fsr1 = 0xC0
        goto    Intret
        bcf     WRT0,a          ;stop writing to block 0
        bsf     RDB0,a          ;allow reading
        goto    Intret
;
Nwb1:   movff   adresh,postinc1 ;copy data to buffer
        movlw   0x00
        cpfseq  fsr1l,a         ;if fsr1 = 0x00
        goto    Intret
        bcf     WRT1,a          ;stop writing to block 1
        bsf     RDB1,a          ;allow reading
        goto    Intret
;
Swb0:   movlw   0x80            ;first byte
        movwf   fsr1l,a
        clrf    fsr1h,a
        movff   adresh,postinc1 ;copy data
        bsf     WRT0,a          ;next time write to block 0
        goto    Intret
;
Swb1:   movlw   0xC0            ;first byte
        movwf   fsr1l,a
        clrf    fsr1h,a
        movff   adresh,postinc1 ;copy data
        bsf     WRT1,a          ;next time write to block 1
        goto    Intret
;
Intret: bsf     adcon0,GO,a     ;start new A/D conversion
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
Start:  clrf    portb,a         ;clear RB3
        movlw   B'11110111'     ;pin RB3 is output
        movwf   trisb,a
        movlw   B'10000000'     ;serial port enabled, pin RC6 is
        movwf   rcsta,a         ; automaticaly configured as output
        movlw   B'00100110'     ;enable transmit
        movwf   txsta,a
        movlw   B'01000000'
        movwf   baudcon,a
        movlw   0x06            ;115200 Bd
        movwf   spbrg,a
        clrf    sspstat,a
        bsf     sspstat,SMP,a   ;disable slew rate control
        movlw   0x03
        movwf   sspadd,a        ;setup 1 MHz I2C clock (819kHz)
        clrf    sspcon2,a
        movlw   B'00101000'     ;I2C Master mode
        movwf   sspcon1,a       ;SSPEN enabled
        bcf     pir1,SSPIF,a    ;clear SSP interrupt flag
        movlw   B'00010001'     ;enable A/D converter module
        movwf   adcon0,a
        movlw   B'00001010'     ;pin RA5 is analog input
        movwf   adcon1,a
        movlw   B'00001101'     ;A/D Conversion Clock = FOSC/16
        movwf   adcon2,a        ;A/D Acquisition Time = 32 / FOSC
        movlw   B'11000001'     ;prescaler 1:4 assigned to TMR0
        movwf   t0con,a
        movlw   B'00100000'
        movwf   intcon,a        ;unmask interrupt from TMR0
;
Main:   btfss   PB0,a           ;test pushbuttons
        goto    Rec             ;Rec button is pressed
        btfss   PB7,a
        goto    Ttpc            ;Transfer button is pressed
        bra     Main
;--------------------------------------------------------------------------
Rec:    bsf     LED,a
        bsf     adcon0,GO,a     ;starts A/D conversion
        clrf    eaddrh,a
        clrf    eaddrl,a
        movlw   B'00000000'
        movwf   reccon,a
        movlw   0x9C            ;TMR0 period = 125usec
        movwf   tmr0l,a
        bcf     intcon,T0IF,a   ;clear timer0 overflow flag
        bsf     intcon,GIE,a    ;enable interrupt
;
Rec2:   btfsc   RDB0,a          ;wait for buffer full flag
        goto    Wrba
        btfsc   RDB1,a
        goto    Wrbb
        bra     Rec2
;
Wrba:   movlw   0x80            ;write bytes 0x80-0xBF to EEPROM
        call    Wree
        bcf     RDB0,a
        bra     Nxtb
Wrbb:   movlw   0xC0            ;write bytes 0xC0-0xFF to EEPROM
        call    Wree
        bcf     RDB1,a
Nxtb:   movlw   0x40            ;eaddrh:l = eaddrh:l + 64
        addwf   eaddrl,F,a
        movlw   0x00
        addwfc  eaddrh,F,a
        tstfsz  eaddrl,a        ;if eaddrh:l = 0x0000 (EEPROM full)
        goto    Rec2
        tstfsz  eaddrh,a
        goto    Rec2
;
        bcf     intcon,GIE,a    ;then disable interrupt
        bcf     LED,a           ;go to main loop
        goto    Main
;--------------------------------------------------------------------------
Ttpc:   movlw   0x00            ;transfer wav file to PC
        call    Rdie            ;read header length
        movwf   bcnt,a
        bsf     LED,a
        movlw   0x01            ;first byte is at address 0x01
        movwf   ieaddr,a
Thtpc:  movf    ieaddr,W,a      ;read from internal EEPROM
        call    Rdie
        call    Send            ;send byte
        incf    ieaddr,F,a
        decfsz  bcnt,F,a
        bra     Thtpc
;
        clrf    eaddrl,a        ;address = 0x0000
        clrf    eaddrh,a
Tetpc:  call    Rdee            ;read I2C EEPROM
        call    Send            ;send byte
        btfss   eaddrh,3,a      ;blinking LED
        bsf     LED,a
        btfsc   eaddrh,3,a
        bcf     LED,a
        movlw   0x01            ;next byte
        addwf   eaddrl,F,a
        movlw   0x00
        addwfc  eaddrh,F,a
        bnc     Tetpc
        bcf     LED,a           ;if eaddrh overflow
        goto    Main            ;go to main loop
;--------------------------------------------------------------------------
; Subroutines :
Send:   btfss   txsta,TRMT,a
        bra     Send
        movwf   txreg,a         ;send data
        return
;
Rdie:   movwf   eeadr,a         ;read from internal EEPROM
        bcf     eecon1,EEPGD,a  ;point to Data memory
        bsf     eecon1,RD,a     ;start read operation
        movf    eedata,W,a      ;W = EEDATA
        return 
;
Sndsb:  bcf     pir1,SSPIF,a    ;this subroutine generates I2C start bit
        bsf     sspcon2,SEN,a   ;generate Start condition
Sndsb2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Sndsb2
        return
;
Sndpb:  bcf     pir1,SSPIF,a    ;this subroutine generates I2C stop bit
        bsf     sspcon2,PEN,a   ;generate Stop condition
Sndpb2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Sndpb2
        return
;
Sndrb:  bcf     pir1,SSPIF,a    ;this subroutine generates I2C restart bit
        bsf     sspcon2,RSEN,a  ;generate Restart condition
Sndrb2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Sndrb2
        return
;
Output: bcf     pir1,SSPIF,a    ;this subroutine serial sends Wreg
        movwf   sspbuf,a
Outpt2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Outpt2
        return
;
Input:  bcf     pir1,SSPIF,a    ;this subroutine serial reads to Wreg
        bsf     sspcon2,RCEN,a  ;initiate reception of byte
Input2: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Input2
        movff   sspbuf,intmp
        bcf     pir1,SSPIF,a    ;clear SSP interrupt flag
        bsf     sspcon2,ACKEN,a ;generate ACK/NO_ACK bit
Input3: btfss   pir1,SSPIF,a    ;check if operation completed
        goto    Input3
        movf    intmp,w,a
        return
;
Rdee:   call    Sndsb           ;read I2C EEPROM from address eaddrh:l
        movlw   B'10100000'     ;control byte - write
        btfsc   eaddrh,7,a      ;bit A15
        iorlw   B'00001000'
        call    Output
        btfss   sspcon2,ACKSTAT,a
        bra     Rdee2           ;ACK bit was sent
        call    Sndpb           ;NO_ACK, stop bit
        nop
        nop
        bra     Rdee
Rdee2:  movf    eaddrh,W,a
        andlw   B'01111111'
        call    Output          ;write address to EEPROM
        movf    eaddrl,W,a
        call    Output          ;write address to EEPROM
        call    Sndrb           ;restart bit
        movlw   B'10100001'     ;control byte - read
        btfsc   eaddrh,7,a      ;bit A15
        iorlw   B'00001000'
        call    Output
        bsf     sspcon2,ACKDT,a ;select NO_ACK bit
        call    Input           ;read data
        movwf   eedatb,a
        call    Sndpb           ;stop bit
        movf    eedatb,W,a
        return
;
Wree:   movwf   fsr0l,a         ;write 64B to EEPROM
        clrf    fsr0h,a
Wree2:  call    Sndsb           ;start bit
        movlw   B'10100000'     ;control byte - write
        btfsc   eaddrh,7,a      ;bit A15
        iorlw   B'00001000'
        call    Output
        btfss   sspcon2,ACKSTAT,a
        bra     Wree3           ;ACK bit was sent
        call    Sndpb           ;NO_ACK, stop bit
        nop
        nop
        bra     Wree2
Wree3:  movf    eaddrh,W,a
        andlw   B'01111111'
        call    Output          ;write address to EEPROM
        movf    eaddrl,W,a
        call    Output          ;write address to EEPROM
        movlw   0x40            ;64 bytes
        movwf   eedatb,a
Wree4:  movf    postinc0,W,a
        call    Output          ;write data to EEPROM
        decfsz  eedatb,F,a
        bra     Wree4
        call    Sndpb           ;stop bit
        return
;**************************************************************************
; end of PVK40 Example : Sound recording
        end
