/*============================================================================

                        PVK40 Example : A/D Converter

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on PHOTO on S9
 4) Switch on B3 LED on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

static bit led @ PORTBIT(PORTB,3);


/*============================================================================

       Program

============================================================================*/

void timer1init(void)
{
 /* initialisation of timer1; */

	T1CKPS1=1; T1CKPS0=1; /* select prescaler by 8 */
	T1OSCEN=0; /* secondary oscillator disabled */
	T1SYNC=0;
	TMR1CS=0; /* clock source - internal clock */
	TMR1ON=1; /* enable timer 1 */
}

void delayus(unsigned char us)
{
  for (;us--;);
}

void adcinit(void)
{
  /* initialisation of AD converter */
  TRISA = 0xFF;	/* all pin of portA are inputs */
  ADCON1 = 0x00;  /* pin RA1 is analog input */
  ADCON0 = 0x49; /* config the converter  */
  delayus(100);
}

void settimer1(unsigned int val)
{
  TMR1ON=0;
  TMR1IF=0;
  TMR1L=val&0xFF;
  TMR1H=val>>8;
  TMR1ON=1;
}

unsigned int adcsample(void)
{

  for (ADCON0|=0x04; ADCON0&0x04; );  /* start and wait for done */

  return ADRESL|(ADRESH<<8);
}


void main(void)
{
  unsigned int speed;

  TRISB=0xF7;
  timer1init();
  adcinit();

  for(;;) {
    speed=adcsample();
    settimer1(speed);
    for(;!TMR1IF;);  /* wait for timer1 overflow */
    led=!led;
  }
}
