/*============================================================================

                        PVK40 Example : Analogue comparator

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on TRIM1 on S9
 4) Switch on TRIM2 on S9
 5) Switch on B0,1,2,4 DISP on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

const unsigned char digit[] = {0xFF,0xFE,0xFD,0xFB,0xEF};
const unsigned char segment[]={0xFF,0xBF,0x89,0xEF,0xC7,0xA3};
unsigned char dispbuffer[4];
unsigned char disppos=0;
unsigned char dispen=0;


/*============================================================================

       Program

============================================================================*/

void dispinit(void)
{
  TRISD=0x00;
  TRISB=0xE8;
}


void disprefresh(void)      /*show the value of an integer j */
{                           /* on one p of the 7 segments display */
  PORTB=digit[0];
  if (!dispen)
    return;
  PORTD=dispbuffer[disppos];
  PORTB=digit[disppos+1];
  disppos=(++disppos)&3;
}


void interrupt timer0(void)
{
	disprefresh();
	T0IF=0;
}


void timer0init(void)
{
 /* initialisation of timer0; */
	PS2=0; PS1=0; PS0=0; /* select prescaler by 2 */
	T0CS=0; /* select timer0 */
	PSA=0;  /* select prescaler for timer0 */
	TMR0=0; /* preload timer0 */
	T0IE=1; /* interrupts are actived */
}


void delayus(unsigned int us)
{
  for (;us--;);
}


void adcinit(void)
{
  /* initialisation of AD converter */
  TRISA = 0xFF;	/* all pin of portA are inputs */
  ADCON1 = 0x80;  /*config in 5/0 */
  ADCON0 = 0x41; /* config the converter  */
}


unsigned int adcsample(unsigned char channel)
{
  ADCON0 = (ADCON0 & 0xc7) | (channel << 3);
  delayus(20);

  for (ADCON0|=0x04; ADCON0&0x04; );  /* start and wait for done */

  return ADRESL|(ADRESH<<8);
}
 

int main(void)
{
  unsigned int trim1;
  unsigned int trim2;

  dispinit();
  timer0init();
  adcinit();
  GIE=1;  /* interrupts are actived */

  dispen=0;
  dispbuffer[0]=segment[0];
  dispbuffer[1]=segment[0];
  dispbuffer[2]=segment[0];
  dispbuffer[3]=segment[0];  
  dispen=1;

  for(;;) {
    trim1=adcsample(2);
    trim2=adcsample(3);
    dispen=0;
    if(trim1>trim2) {
      dispbuffer[0]=segment[3];
      dispbuffer[1]=segment[2];
    }
    if(trim1<trim2) {
      dispbuffer[0]=segment[5];
      dispbuffer[1]=segment[4];
    }
    if(trim1==trim2) {
      dispbuffer[0]=segment[1];
      dispbuffer[1]=segment[1];
    } 
    dispen=1;
    delayus(2000);
  }
}
