/*============================================================================

                        PVK40 Example : Sound-level-meter

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on E0 SEGM9 on S9
 4) Switch on E1 SEGM10 on S9
 5) Switch on A5 MIC on S9
 6) Switch on C1 BARGRAPH on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

const unsigned int level[] = {45,64,91,128,181,256,313,512,640,762};

/*============================================================================

       Program

============================================================================*/

void portinit(void)
{
  PORTC=0x00;
  PORTD=0xFF;
  PORTE=0xFF;
  TRISC=0xFD;
  TRISD=0x00;
  TRISE=0x0C;
}

void adcinit(void)
{  /* initialisation of AD converter */
  TRISA = 0xFF;	/* all pin of portA are inputs */
  ADCON1 = 0x82;  /*config in 5/0 */
  ADCON0 = 0x61; /* config the converter  */
}

void timer0init(void)
{ /* initialisation of timer0; */
	PS2=0; PS1=0; PS0=0; /* select prescaler by 2 */
	T0CS=0; /* select timer0 */
	PSA=0;  /* select prescaler for timer0 */
	TMR0=0; /* preload timer0 */
}

unsigned int adcsample(void)
{
  for (ADCON0|=0x04; ADCON0&0x04; );  /* start and wait for done */
  return ADRESL|(ADRESH<<8);
}

void bargr_disp(unsigned int value)
{
  unsigned int disp, mask;
  char n;

  disp=0xFFFF;
  mask=0xFFFE;
  for(n=0;n<10;n++) {
    if(value>level[n])
      disp&=mask;
    mask=(mask<<1)|0x0001;
  }
  PORTD=disp&0xFF;
  PORTE=(disp>>8)&0x03;
}

int main(void)
{
  unsigned int peak,down,sample;
  unsigned char i;

  portinit();
  adcinit();
  timer0init();

  for(;;) {
    peak=0x0000;    /* sample 255 samples and find min/max values */
    down=0xFFFF;
    for(i=0xFF;i>0;i--) {  /* 255 samples */
      sample=adcsample();
      while(!T0IF) ;     /* wait for timer0 overflow */
      T0IF=0;
      if(sample>peak)  /* compare with limits */
        peak=sample;
      if(sample<down)
        down=sample;
    }
  bargr_disp(peak-down);
  }
}
