/*============================================================================

                        PVK40 Example : Piano

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on C2 BUZZER on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>

/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

const unsigned int voltbl[] = {0x14,0x10,0x10,0x10,0x0C,0x0C,0x0C,0x08};
const unsigned char freqtbl[]={0x9E,0xA9,0xB2,0xB7,0xBF,0xC6,0xCC,0xCF};

static bit pushbutton0 @ PORTBIT(PORTD,0);
static bit pushbutton1 @ PORTBIT(PORTD,1);
static bit pushbutton2 @ PORTBIT(PORTD,2);
static bit pushbutton3 @ PORTBIT(PORTD,3);
static bit pushbutton4 @ PORTBIT(PORTD,4);
static bit pushbutton5 @ PORTBIT(PORTD,5);
static bit pushbutton6 @ PORTBIT(PORTD,6);
static bit pushbutton7 @ PORTBIT(PORTD,7);


/*============================================================================

       Program

============================================================================*/

void pwminit(void)
{
  CCPR1H=0x00;
  CCPR1L=0x00;
  CCP1CON=0x0F;
  TMR2=0x00;
  TMR2ON=1;
  T2CKPS1=1; T2CKPS0=1; /* prescaler 1:16 */
  PR2=0xFF;
  TRISC&=0xFB;
}

void tone(unsigned int volume, unsigned char freq)
{
  if (volume&1)
    CCP1Y=1;
  else
    CCP1Y=0;
  if (volume&2)
    CCP1X=1;
  else
    CCP1X=0;
  CCPR1L=(volume>>2)&0xFF;
  PR2=~freq;
}

int main(void)
{
  char btn_index;

  pwminit();
  for(;;) {
    btn_index=8;
    if(!pushbutton0)
      btn_index=0;
    if(!pushbutton1)
      btn_index=1;
    if(!pushbutton2)
      btn_index=2;
    if(!pushbutton3)
      btn_index=3;
    if(!pushbutton4)
      btn_index=4;
    if(!pushbutton5)
      btn_index=5;
    if(!pushbutton6)
      btn_index=6;
    if(!pushbutton7)
      btn_index=7;

    if(btn_index==8)
      tone(0,0);
    else
      tone(voltbl[btn_index],freqtbl[btn_index]);
  }
}
